/***********************************************************************

Vorosketch User Distances Demo by Herman Haverkort, 10 August 2022

************************************************************************

Copyright 2022 Herman Johannes Haverkort

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This file is designed to be used with Vorosketch v0.16 BETA; it might 
work with later versions too. New versions may be available from 
http://herman.haverkort.net.


***********************************************************************/


#define MAKEUSERDISTANCE \
if (dname == "mydistance") return new MyDistance();\
if (dname == "quotient") return new QuotientDistance();\
if (dname == "karlsruhedilation") return new KarlsruheDilationDistance();

class MyDistance: public PointDistance
{
    public:
  virtual const string description() const
  {
    return "My Distance (default Euclidean)";
  }

  virtual DOUBLE operator()(Point& p, const Site& s) const
  {
    return (p - s.points[0]).r();
  }
};


class QuotientDistance: public PointDistance
{
    public:
  virtual const string description() const
  {
    return "quotient distance (y/x)";
  }

  virtual DOUBLE operator()(Point& p, const Site& s) const
  {
    if (p == s.points[0]) return 0;
    if (p.x() == s.points[0].x()) return INF;
    return abs(p.y() - s.points[0].y()) / abs(p.x() - s.points[0].x());
  }
};


class KarlsruheDilationDistance: public PointDistance
{
    public:
  virtual const string description() const
  {
    return "Karlsruhe Dilation";
  }

  virtual DOUBLE operator()(Point& p, const Site& s) const
  {
    DOUBLE angleDistance = safeAcos((p * s.points[0]) / (p.r() * s.points[0].r()));
    DOUBLE euclidean = (p - s.points[0]).r();
    if (euclidean == 0) return 0;
    DOUBLE karlsruhe = (angleDistance > 2) ?
      (p.r() + s.points[0].r()) :
      ( angleDistance * min(p.r(), s.points[0].r()) + 
        abs(p.r() - s.points[0].r()));
    return sqrt(min(sqr(karlsruhe / euclidean), (DOUBLE) 2.0) - (DOUBLE) 1.0);
  }
};
